# -*- coding: utf-8 -*-
from plone import api
from plone.app.imagecropping.events import CroppingInfoChangedEvent
from plone.app.imagecropping.events import CroppingInfoRemovedEvent
from plone.app.imagecropping.interfaces import ICroppingInfoChangedEvent
from plone.app.imagecropping.interfaces import ICroppingInfoRemovedEvent
from plone.app.imagecropping.testing import IMAGECROPPING_FUNCTIONAL
from plone.app.imagecropping.tests import dummy_named_blob_png_image
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.testing.z2 import Browser
from zope.component import getGlobalSiteManager

import transaction
import unittest


class EditorTestCase(unittest.TestCase):

    layer = IMAGECROPPING_FUNCTIONAL

    def setUp(self):
        self.app = self.layer['app']
        self.portal = self.layer['portal']

        # setup testbrowser
        self.browser = Browser(self.app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            'Authorization',
            'Basic {0:s}:{1:s}'.format(TEST_USER_NAME, TEST_USER_PASSWORD)
        )
        self.createSingleImageType()

    def createSingleImageType(self):
        # create test image as testuser
        self.portal.invokeFactory(
            'Image',
            'testimage',
            title=u'I\'m a testing Image'
        )
        self.img = self.portal['testimage']
        self.img.image = dummy_named_blob_png_image()
        self.img.reindexObject()
        transaction.commit()

    def test_editview_crop(self, check_assert=True):
        scale_name = 'mini'
        request = self.layer['request']
        request.form.update({
            'x': 1.0, 'y': 2.7, 'width': 9.6, 'height': 5.7,
            'fieldname': 'image',
            'scale': scale_name, 'form.button.Save': '1'})
        cropview = api.content.get_view('crop-image', self.img, request)
        result = cropview()
        self.assertEqual(result, 'OK')

        def get_cropped_scale(scales):
            return [s for s in scales if s['id'] == scale_name][0]

        cropedit = api.content.get_view('croppingeditor', self.img, request)
        cropedit()
        cropped_scale = get_cropped_scale(cropedit.scales_info('image'))
        if check_assert:
            self.assertEqual(cropped_scale['is_cropped'], True)

    def test_editview_remove(self, check_assert=True):
        scale_name = 'mini'
        request = self.layer['request']
        request.form.update({
            'remove': 1,
            'fieldname': 'image',
            'scale': scale_name, 'form.button.Save': '1'})
        cropview = api.content.get_view('crop-image', self.img, request)
        result = cropview()
        self.assertEqual(result, 'OK')

    def test_events(self):
        sm = getGlobalSiteManager()
        firedEvents = []

        def recordEvent(event):
            firedEvents.append(event.__class__)

        sm.registerHandler(recordEvent, (ICroppingInfoChangedEvent,))
        sm.registerHandler(recordEvent, (ICroppingInfoRemovedEvent,))

        # do some cropping and removing
        self.test_editview_crop(check_assert=False)
        self.test_editview_remove(check_assert=False)

        self.assertItemsEqual(firedEvents, [
            CroppingInfoChangedEvent,
            CroppingInfoRemovedEvent,
        ])
        sm.unregisterHandler(recordEvent, (ICroppingInfoChangedEvent,))
        sm.unregisterHandler(recordEvent, (ICroppingInfoRemovedEvent,))

    def tearDown(self):
        api.content.delete(self.portal.testimage)



# -*- coding: utf-8 -*-
from zope.component.interfaces import IObjectEvent
from zope.interface.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class IPloneAppImagecroppingLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""


class IImageCroppingMarker(Interface):
    """Generic marker interface for image cropping support.

    do not use directly, use the more special interfaces in dx or at modules
    """


class IImageCroppingUtils(Interface):
    """Methods to help the cropping editor views.
       The implementation is different for Archetype and Dexterity context.
    """

    def image_fields():
        """Returns all image fields"""

    def image_field_names():
        """Returns the names of all image fields"""

    def get_image_field(fieldname):
        """Returns the image field"""

    def get_image_label(fieldname):
        """Returns the label of the image field"""

    def get_image_data(fieldname):
        """Returns the image data"""

    def get_image_size(fieldname):
        """Returns the original image size:

           (100, 200)
        """

    def save_cropped(fieldname, scale, image_file):
        """ Save the cropped iamge under the name of the selected scale in
            plone.scale.storage.AnnotationStorage, so that it is available
            in plone.app.imaging @@images view
        """


class ICroppingInfoChangedEvent(IObjectEvent):
    """ event after cropping information has changed """


class ICroppingInfoRemovedEvent(IObjectEvent):
    """ event after cropping information has changed """


# seems unused
# class ICroppedImageScaling(IImageScaling):
#     """ marker for @@images overrides """


# class ICroppedBlobImageField(IBlobImageField):
#     """ marker for coppable blob image field adapters """



# -*- coding: utf-8 -*-
from plone.app.imagecropping.browser.settings import ISettings
from plone.app.imagecropping.testing import IMAGECROPPING_INTEGRATION
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.registry.interfaces import IRegistry
from zope.component import getMultiAdapter
from zope.component import getUtility

import unittest


class ControlPanelTestCase(unittest.TestCase):

    layer = IMAGECROPPING_INTEGRATION

    def setUp(self):
        self.portal = self.layer['portal']
        self.controlpanel = self.portal['portal_controlpanel']
        self.qi_tool = self.portal['portal_quickinstaller']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_controlpanel_has_view(self):
        view = getMultiAdapter(
            (self.portal, self.portal.REQUEST),
            name='imagecropping-settings'
        )
        view = view.__of__(self.portal)
        self.assertTrue(view())

    def test_controlpanel_view_is_protected(self):
        from AccessControl import Unauthorized
        logout()
        self.assertRaises(
            Unauthorized,
            self.portal.restrictedTraverse,
            '@@imagecropping-settings'
        )


class RegistryTestCase(unittest.TestCase):

    layer = IMAGECROPPING_INTEGRATION

    def setUp(self):
        self.portal = self.layer['portal']
        self.qi_tool = self.portal['portal_quickinstaller']
        self.registry = getUtility(IRegistry)
        self.settings = self.registry.forInterface(ISettings)

    def test_available_sections_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'large_size'))
        self.assertEqual(self.settings.large_size, u'768:768')

    def test_default_section_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'min_size'))
        self.assertEqual(self.settings.min_size, u'50:50')

    def test_records_removed_on_uninstall(self):
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.qi_tool.uninstallProducts(products=['plone.app.imagecropping'])

        BASE_REGISTRY = \
            'plone.app.imagecropping.browser.settings.ISettings.{0:s}'
        records = (
            BASE_REGISTRY.format('large_size'),
            BASE_REGISTRY.format('min_size'),
        )

        for r in records:
            self.assertNotIn(r, self.registry)



# -*- coding: utf-8 -*-
from plone.app.imagecropping import PAI_STORAGE_KEY
from plone.app.imagecropping.interfaces import IImageCroppingMarker
from plone.app.imagecropping.interfaces import IImageCroppingUtils
from zope.annotation.interfaces import IAnnotations
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.globalrequest import getRequest
from zope.lifecycleevent.interfaces import IObjectCopiedEvent


@adapter(IImageCroppingMarker, IObjectCopiedEvent)
def apply_crops_after_copy(context, event):
    crops = IAnnotations(event.original).get(PAI_STORAGE_KEY)
    if not crops:
        return
    croputils = IImageCroppingUtils(context)
    request = getRequest()
    cropper = getMultiAdapter((context, request), name='crop-image')
    for fieldname in croputils.image_field_names():
        for crop_key in crops:
            if crop_key.startswith(fieldname):
                scalename = crop_key[len(fieldname) + 1:]
                cropper._crop(fieldname, scalename, crops[crop_key])



# -*- coding: utf-8 -*-
from plone.app.imagecropping.interfaces import IImageCroppingMarker
from plone.app.imagecropping.interfaces import IImageCroppingUtils
from plone.app.imaging.utils import getAllowedSizes
from plone.behavior.interfaces import IBehaviorAssignable
from plone.namedfile.interfaces import IImage
from plone.namedfile.interfaces import IImageScaleTraversable
from plone.scale.scale import scaleImage
from plone.scale.storage import AnnotationStorage
from zope.component import adapter
from zope.interface import implementer
from zope.schema import getFieldsInOrder

import time


class IImageCroppingDX(IImageScaleTraversable, IImageCroppingMarker):
    """Image cropping support marker interface for NamedFile/DX types
    """


def _millis():
    return int(time.time() * 1000)


@implementer(IImageCroppingUtils)
@adapter(IImageScaleTraversable)
class CroppingUtilsDexterity(object):
    """Dexterity variant of scaling adapter
    """

    def __init__(self, context):
        self.context = context

    def _all_fields(self):
        type_info = self.context.getTypeInfo()
        if type_info is None:
            return
        schema = type_info.lookupSchema()
        for field in getFieldsInOrder(schema):
            yield field
        behavior_assignable = IBehaviorAssignable(self.context)
        if behavior_assignable:
            for behavior in behavior_assignable.enumerateBehaviors():
                for field in getFieldsInOrder(behavior.interface):
                    yield field

    def _image_field_values(self):
        for fieldname, field in self._all_fields():
            value = getattr(self.context, fieldname, None)
            if value and IImage.providedBy(value):
                yield (fieldname, value)

    def image_fields(self):
        """ read interface
        """
        return [info[1] for info in self._image_field_values()]

    def image_field_names(self):
        """ read interface
        """
        return [info[0] for info in self._image_field_values()]

    def get_image_field(self, fieldname):
        """ read interface
        """
        return getattr(self.context, fieldname, None)

    def get_image_label(self, fieldname):
        for name, field in self._all_fields():
            if name == fieldname:
                return field.title
        return fieldname

    def get_image_data(self, fieldname):
        """ read interface
        """
        field = self.get_image_field(fieldname)
        return field.data

    def get_image_size(self, fieldname):
        """ read interface
        """
        field = self.get_image_field(fieldname)
        image_size = field.getImageSize()
        return image_size

    def save_cropped(self, fieldname, scale, image_file):
        """ see interface
        """
        sizes = getAllowedSizes()
        w, h = sizes[scale]

        def crop_factory(fieldname, **parameters):
            result = scaleImage(image_file.read(), **parameters)
            if result is not None:
                data, format, dimensions = result
                mimetype = 'image/{0:s}'.format(format.lower())
                field = self.get_image_field(fieldname)
                value = field.__class__(
                    data,
                    contentType=mimetype,
                    filename=field.filename
                )
                value.fieldname = fieldname
                return value, format, dimensions

        # call storage with actual time in milliseconds
        # this always invalidates old scales
        storage = AnnotationStorage(self.context, _millis)

        # We need to pass direction='thumbnail' since this is the default
        # used by plone.namedfile.scaling, also for retrieval of scales.
        # Otherwise the key under which the scaled and cropped image is
        # saved in plone.scale.storage.AnnotationStorage will not match the
        # key used for retrieval (= the cropped scaled image will not be
        # found)
        storage.scale(
            factory=crop_factory,
            direction='thumbnail',
            fieldname=fieldname,
            width=w,
            height=h,
        )



# -*- coding: utf-8 -*-
from cStringIO import StringIO
from persistent.dict import PersistentDict
from plone import api
from plone.app.imagecropping import PAI_STORAGE_KEY
from plone.app.imagecropping.events import CroppingInfoChangedEvent
from plone.app.imagecropping.events import CroppingInfoRemovedEvent
from plone.app.imagecropping.interfaces import IImageCroppingUtils
from plone.scale.storage import AnnotationStorage
from Products.Five.browser import BrowserView
from z3c.caching.purge import Purge
from zope.annotation.interfaces import IAnnotations
from zope.event import notify

import logging
import PIL.Image


logger = logging.getLogger(__name__)


class CroppingView(BrowserView):

    DEFAULT_FORMAT = 'PNG'

    def __call__(self, **kw):
        form = self.request.form
        fieldname = form['fieldname']
        scale_id = form['scale']
        if 'remove' in form:
            self._remove(fieldname, scale_id)
            return 'OK'

        box = (
            int(round(float(form['x']))),
            int(round(float(form['y']))),
            int(round(float(form['x']) + float(form['width']))),
            int(round(float(form['y']) + float(form['height']))),
        )
        self._crop(form['fieldname'], form['scale'], box)
        return 'OK'

    def _crop(self, fieldname, scale, box):
        """switch between dexterity and Archetypes
        """
        croputils = IImageCroppingUtils(self.context)
        data = croputils.get_image_data(fieldname)

        original_file = StringIO(data)
        image = PIL.Image.open(original_file)
        image_format = image.format or self.DEFAULT_FORMAT

        cropped_image = image.crop(box)
        cropped_image_file = StringIO()
        cropped_image.save(cropped_image_file, image_format, quality=100)
        cropped_image_file.seek(0)

        croputils.save_cropped(fieldname, scale, cropped_image_file)

        # store crop information in annotations
        self._store(fieldname, scale, box)

        notify(CroppingInfoChangedEvent(self.context))
        # Purge caches if needed
        notify(Purge(self.context))

    def _key(self, fieldname, scale):
        return '{0:s}_{1:s}'.format(fieldname, scale)

    def _remove(self, fieldname, scale):
        # remove info from annotation
        key = self._key(fieldname, scale)
        if key in self._storage.keys():
            del self._storage[key]

        # remove saved scale
        scale_storage = AnnotationStorage(self.context)
        image_scales = api.content.get_view(
            'images',
            self.context,
            self.request
        )
        image_scale = image_scales.scale(fieldname, scale=scale)
        del scale_storage[image_scale.uid]

        notify(CroppingInfoRemovedEvent(self.context))
        # Purge caches if needed
        notify(Purge(self.context))

    @property
    def _storage(self):
        return IAnnotations(
            self.context
        ).setdefault(
            PAI_STORAGE_KEY,
            PersistentDict()
        )

    def _store(self, fieldname, scale, box):
        key = self._key(fieldname, scale)
        self._storage[key] = box

    def _read(self, fieldname, scale):
        key = self._key(fieldname, scale)
        return self._storage.get(key)



# -*- coding: utf-8 -*-
from .scaling import ScalingOverrides
from plone.app.imaging.scaling import ImageScaling as BaseImageScaling


class ImageScalingAT(ScalingOverrides, BaseImageScaling):

    def modified(self):
        """we overwrite the default method that would return the modification
        time of the context,
        to return a way back modification time in case the currently requested
        scale is a cropped scale. (so plone.scale does not create a new scale
        w/o cropping information
        """
        if self._allow_rescale:
            return super(ImageScalingAT, self).modified()
        else:
            return 1

    def scale(self, fieldname=None, scale=None, height=None, width=None,
              **parameters):
        self._need_rescale(fieldname, scale)
        # if direction is 'down' and we have a cropped scale
        # deliver it instead of standard 'down' scale
        p_dir = parameters.get('direction', '')
        if p_dir == 'down' and not self._allow_rescale:
            del parameters['direction']
        return super(ImageScalingAT, self).scale(
            fieldname, scale, height, width, **parameters)



# -*- coding: utf-8 -*-
from plone.app.imagecropping.interfaces import ICroppingInfoChangedEvent
from plone.app.imagecropping.interfaces import ICroppingInfoRemovedEvent
from zope.component.interfaces import ObjectEvent
from zope.interface import implementer


@implementer(ICroppingInfoChangedEvent)
class CroppingInfoChangedEvent(ObjectEvent):
    """ """


@implementer(ICroppingInfoRemovedEvent)
class CroppingInfoRemovedEvent(ObjectEvent):
    """ """



# -*- coding: utf-8 -*-
from plone.app.imagecropping.dx import IImageCroppingDX


class IImageCroppingBehavior(IImageCroppingDX):
    """
    Dumb marker behavior to enable imagecropping as a behavior on your own
    content types.
    """



# -*- coding: utf-8 -*-
from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles(object):

    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller"""
        return [
            'plone.app.imagecropping:uninstall',
        ]



# -*- coding: utf-8 -*-
from plone.app.imagecropping import imagecroppingMessageFactory as _
from plone.app.imaging.utils import getAllowedSizes
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.z3cform import layout
from z3c.form import form
from zope import schema
from zope.interface import implementer
from zope.interface import Interface
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary


@implementer(IVocabularyFactory)
class SizesVocabulary(object):

    def __call__(self, context):
        allowed_sizes = getAllowedSizes()
        size_names = allowed_sizes and allowed_sizes.keys() or []
        return SimpleVocabulary.fromValues(size_names)


class ISettings(Interface):
    ' Define settings data structure '

    large_size = schema.TextLine(
        title=_(u'Crop Editor Large Size'),
        description=_(u'width:height'),
        required=False,
        default=u'768:768',
    )

    min_size = schema.TextLine(
        title=_(u'Minimum Crop Area Size'),
        description=_(u'width:height'),
        required=False,
        default=u'50:50',
    )

    constrain_cropping = schema.Bool(
        title=_(u'Enable to constrain croppable scales'),
        description=_(u'Enable to reduce the scales shown for cropping in the '
                      u'list of scales with crop support.'),
        default=False,
        required=False,
    )

    cropping_for = schema.List(
        title=_(u'List of scales with crop support'),
        description=_(u'Select the scales with cropping support enabled. Only '
                      u'active if enabled with checkbox.'),
        required=False,
        default=[],
        value_type=schema.Choice(
            vocabulary='plone.app.imagecropping.all_sizes'),
    )

    """XXX: not implemented right now
    auto_cropping_for = schema.List(
        title=_(u'Scales with auto crop'),
        description=_(u'Scales to be auto cropped in center of image.'),
        required=False,
        default=[],
        value_type=Choice(
            vocabulary='plone.app.imagecropping.all_sizes'),
    )
    """


class SettingsEditForm(RegistryEditForm):
    """ Define form logic
    """
    form.extends(RegistryEditForm)
    schema = ISettings
    label = _(u'Image Cropping Settings')


SettingsView = layout.wrap_form(SettingsEditForm, ControlPanelFormWrapper)



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from plone import api
from plone.app.imagecropping.testing import IMAGECROPPING_FUNCTIONAL

import unittest


class TestSetup(unittest.TestCase):
    """Test that plone.app.imagecropping is properly installed."""

    layer = IMAGECROPPING_FUNCTIONAL

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')

    def test_product_installed(self):
        """Test if plone.app.imagecropping is installed."""
        self.assertTrue(self.installer.isProductInstalled(
            'plone.app.imagecropping'))

    def test_browserlayer(self):
        """Test that IPloneAppImagecroppingLayer is registered."""
        from plone.app.imagecropping.interfaces import (
            IPloneAppImagecroppingLayer)
        from plone.browserlayer import utils
        self.assertIn(IPloneAppImagecroppingLayer, utils.registered_layers())

    def test_controlpanel(self):
        actions = [
            a.getAction(self)['id']
            for a in self.portal['portal_controlpanel'].listActions()
        ]
        self.assertIn(
            'imagecropping.settings',
            actions,
            'control panel was not removed'
        )


class TestUninstall(unittest.TestCase):

    layer = IMAGECROPPING_FUNCTIONAL

    def setUp(self):
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')
        self.installer.uninstallProducts(products=['plone.app.imagecropping'])

    def test_product_uninstalled(self):
        """Test if plone.app.imagecropping is cleanly uninstalled."""
        self.assertFalse(
            self.installer.isProductInstalled('plone.app.imagecropping')
        )

    def test_browserlayer_removed(self):
        """Test that IPloneAppImagecroppingLayer is removed."""
        from plone.app.imagecropping.interfaces import \
            IPloneAppImagecroppingLayer
        from plone.browserlayer import utils

        self.assertNotIn(
            IPloneAppImagecroppingLayer,
            utils.registered_layers()
        )

    def test_controlpanel_removed(self):
        # controlpanel configlets are not removed but set to invisible
        action = [
            a.getAction(self)
            for a in self.portal['portal_controlpanel'].listActions()
            if a.getAction(self)['id'] == 'imagecropping.settings'
        ][0]
        self.assertFalse(action['visible'])



# -*- coding: utf-8 -*-
from operator import itemgetter
from plone import api
from plone.app.imagecropping.browser.settings import ISettings
from plone.app.imagecropping.interfaces import IImageCroppingUtils
from plone.app.imaging.utils import getAllowedSizes
from plone.registry.interfaces import IRegistry
from Products.Five.browser import BrowserView
from zope.component._api import getUtility


class CroppingEditor(BrowserView):
    """ Cropping Editor View """

    def __init__(self, context, request):
        super(CroppingEditor, self).__init__(context, request)
        request.set('disable_plone.leftcolumn', 1)
        request.set('disable_plone.rightcolumn', 1)

    @property
    def _croputils(self):
        return IImageCroppingUtils(self.context)

    @property
    def _editor_settings(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISettings)
        return settings

    @property
    def show_cropping(self):
        """returns True if there are any croppable scales on any of the fields
        """
        for fieldname in self.image_field_names:
            for value in self._scales(fieldname):
                return True
        return False

    def _min_size(self, image_size, scale_size):
        """ we need lower min-sizes if the image is smaller than the scale """
        width = scale_size[0]
        height = scale_size[1]
        if width > image_size[0]:
            ratio = float(image_size[0]) / float(width)
            width = image_size[0]
            height = float(height) * ratio
        if height > image_size[1]:
            ratio = float(image_size[1]) / float(height)
            height = image_size[1]
            width = float(width) * ratio
        return (int(round(width)), int(round(height)))

    def _initial_size(self, image_size, scale_size):
        """we need a best fit centered preselection to make editors life
        better.
        """
        ix, iy = map(float, image_size)

        # aspect ratio of original
        if iy > 0:
            ir = ix / iy
        else:
            ir = 1

        sx, sy = map(float, scale_size)

        # aspect ratio of scale
        if sy > 0:
            sr = sx / sy
        else:
            sr = 1

        # scale up to bounds
        if ir > sr:
            rx1, ry1 = ix * sr / ir, iy
        else:
            rx1, ry1 = ix, iy * ir / sr

        rx0, ry0 = 0, 0

        # center box
        if rx1 < ix:
            deltax = ix - rx1
            rx0 = deltax / 2
            rx1 = rx1 + deltax / 2
        if ry1 < iy:
            deltay = iy - ry1
            ry0 = deltay / 2
            ry1 = ry1 + deltay / 2

        # round to int
        rx0, ry0 = int(round(rx0)), int(round(ry0))
        rx1, ry1 = int(round(rx1)), int(round(ry1))
        return rx0, ry0, rx1, ry1

    def _scale_info(self, fieldname, scale_id, target_size, true_size):
        scale = dict()
        scale['id'] = scale_id
        scale['title'] = scale_id

        # scale config
        min_width, min_height = self._min_size(true_size, target_size)

        # lookup saved crop info
        current_box = api.content.get_view(
            'crop-image',
            self.context,
            self.request
        )._read(fieldname, scale_id)

        if current_box is None:
            current_box = self._initial_size(true_size, target_size)
            scale['is_cropped'] = False
        else:
            scale['is_cropped'] = True

        # images original dimensions
        scale['true_width'] = true_size[0]
        scale['true_height'] = true_size[1]

        # images target dimensions
        scale['target_width'] = target_size[0]
        scale['target_height'] = target_size[1]

        # current selected crop
        scale['current'] = {
            'x': current_box[0],
            'y': current_box[1],
            'w': current_box[2] - current_box[0],
            'h': current_box[3] - current_box[1],
        }
        scale['aspect_ratio'] = '{0:.2f}'.format(
            float(target_size[0]) / float(target_size[1])
        )
        scale['can_scale'] = (
            target_size[0] <= true_size[0] and
            target_size[1] <= true_size[1]
        )
        return scale

    def _scales(self, fieldname):
        constrain_cropping = self._editor_settings.constrain_cropping
        cropping_for = self._editor_settings.cropping_for
        sizes_iterator = sorted(
            getAllowedSizes().iteritems(),
            key=itemgetter(1)
        )
        for scale_id, target_size in sizes_iterator:
            if constrain_cropping and scale_id not in cropping_for:
                continue
            yield scale_id, target_size

    def scales_info(self, fieldname):
        """Returns information to initialize pattern for all available scales
           on the current content with the given fieldname and interface."""
        true_size = self._croputils.get_image_size(fieldname)
        for scale_id, target_size in self._scales(fieldname):
            yield self._scale_info(
                fieldname,
                scale_id,
                target_size,
                true_size
            )

    @property
    def image_field_names(self):
        return self._croputils.image_field_names()

    def icon_url(self, fieldname):
        scales = api.content.get_view('images', self.context, self.request)
        scaled_img = scales.scale(
            fieldname,
            scale='icon',
            direction='keep',
        )
        return scaled_img and scaled_img.url or ''

    def original_url(self, fieldname):
        """Returns the url to the unscaled image"""
        url = self.context.absolute_url()
        return '{0}/images/{1}'.format(url, fieldname)

    def field_label(self, fieldname):
        return self._croputils.get_image_label(fieldname)



# -*- coding: utf-8 -*-
from cStringIO import StringIO
from os.path import dirname
from PIL import Image
from plone.namedfile.file import NamedBlobImage


TEST_IMAGE_FILE = '/'.join([dirname(__file__), u'plone-logo.png'])


def dummy_named_blob_png_image():
    return NamedBlobImage(
        data=open(TEST_IMAGE_FILE, 'r').read(),
        filename=TEST_IMAGE_FILE
    )


def dummy_named_blob_jpg_image():
    img = Image.open(TEST_IMAGE_FILE)
    out = StringIO()
    img.save(out, format='JPEG', quality=75)
    out.seek(0)
    bi = NamedBlobImage(data=out.getvalue())
    out.close()
    return bi



# -*- coding: utf-8 -*-
from .scaling import ScalingOverrides
from plone.namedfile.scaling import ImageScaling as NFImageScaling


class ImageScalingDX(ScalingOverrides, NFImageScaling):
    """ Override plone.namedfile scaling view

        This view checks, if image crops are available and
        prevents rescaling in this case.
    """

    def modified(self):
        """We overwrite the default method that would return the
           modification time of the context, to return a way back
           modification time in case the currently requested scale
           is a cropped scale. (so plone.scale does not create a
           new scale w/o cropping information.
        """
        if self._allow_rescale:
            return super(ImageScalingDX, self).modified()
        else:
            return 1

    def scale(self, fieldname=None, scale=None, height=None, width=None,
              direction='thumbnail', **parameters):
        self._need_rescale(fieldname, scale)
        # if direction is 'down' and we have a cropped scale
        # deliver it instead of standard 'down' scale
        if direction == 'down' and not self._allow_rescale:
            direction = 'thumbnail'
        return super(ImageScalingDX, self).scale(
            fieldname, scale, height, width, direction, **parameters)



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import login
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.testing import z2


class PloneAppImagecropping(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import plone.app.imagecropping
        self.loadZCML('testing.zcml', package=plone.app.imagecropping)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'plone.app.imagecropping:testing')
        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)
        portal.portal_workflow.setDefaultChain('one_state_workflow')
        portal.invokeFactory(
            'Folder',
            id='acceptance-test-folder',
            title=u'Test Folder'
        )


IMAGECROPPING = PloneAppImagecropping()
IMAGECROPPING_INTEGRATION = IntegrationTesting(
    bases=(IMAGECROPPING, ),
    name='plone.app.imagecropping:Integration'
)
IMAGECROPPING_FUNCTIONAL = FunctionalTesting(
    bases=(IMAGECROPPING, ),
    name='plone.app.imagecropping:Functional')
IMAGECROPPING_ROBOT = FunctionalTesting(
    bases=(
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        IMAGECROPPING,
        z2.ZSERVER_FIXTURE
    ),
    name='plone.app.imagecropping:Robot'
)



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup


version = '2.0b2'

setup(
    name='plone.app.imagecropping',
    version=version,
    description='Crops Images in Plone manually using cropper JS library',
    long_description='\n\n'.join([
        open('README.rst').read(),
        open('CONTRIBUTORS.rst').read(),
        open('CHANGES.rst').read(),
    ]),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Plone',
        'Framework :: Plone :: 5.0',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Operating System :: OS Independent',
        'Programming Language :: JavaScript',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords='plone image crop',
    author='Plone Collective',
    author_email='plone-developers@lists.sourceforge.net',
    url='https://github.com/collective/plone.app.imagecropping',
    license='GPLv2',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['plone', 'plone.app'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'Products.CMFPlone>5.0.2',
        'setuptools',
    ],
    extras_require={
        'test': [
            'plone.app.testing',
            'plone.app.robotframework[debug]',
        ],
    },
    entry_points="""
    # -*- Entry points: -*-
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



# -*- coding: utf-8 -*-
from plone.app.imagecropping import PRODUCT_NAME
from plone.app.imagecropping.browser.settings import ISettings
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.utils import getToolByName
from zope.component import queryUtility

import logging


logger = logging.getLogger(__name__)

PROFILE_ID = 'profile-{0:s}:default'.format(PRODUCT_NAME)


def _cookResources(context):
    jstool = getToolByName(context, 'portal_javascripts')
    jstool.cookResources()
    csstool = getToolByName(context, 'portal_css')
    csstool.cookResources()


def to_0004(context):
    """search for iimagecropping interface in catalog and reindex those objects
    so they implment our new interfaces
    """
    iface = 'plone.app.imagecropping.interfaces.IImageCropping'
    cat = getToolByName(context, 'portal_catalog')

    # this consumes a lot of memory if we have too many objects to
    # update; we better use a generator to reduce memory usage and
    # avoid restarts on instances running with supervisor's memmon
    results = (b.getObject() for b in cat(object_provides=iface))
    for obj in results:
        obj.reindexObject(idxs=['object_provides'])


def migrate0002to0003(context):
    setup_tool = getToolByName(context, 'portal_setup')
    setup_tool.runImportStepFromProfile(PROFILE_ID, 'plone.app.registry')

    registry = queryUtility(IRegistry)
    settings = registry.forInterface(ISettings, check=False)
    if not hasattr(settings, 'constrain_cropping'):
        settings.constrain_cropping = False
    if not hasattr(settings, 'cropping_for'):
        settings.cropping_for = []
    if not hasattr(settings, 'default_cropping_for'):
        settings.default_cropping_for = []
    logger.info('Registry cleanup operation performed')
    logger.info('Migrated to profile version 0003')
